==> cloud-3.0.0-rc1.tar.gz: cloud/modules/cloud_service_providers/openstack/tests/src/Traits/OpenStackTestFormDataTrait.php <==
<?php

namespace Drupal\Tests\openstack\Traits;

use Drupal\Component\Utility\Random;
use Drupal\Tests\aws_cloud\Traits\AwsCloudTestFormDataTrait;
use Drupal\Tests\cloud\Functional\Utils;

/**
 * The trait creating form data for openstack testing.
 */
trait OpenStackTestFormDataTrait {

  use AwsCloudTestFormDataTrait;

  /**
   * OPENSTACK_SECURITY_GROUP_REPEAT_COUNT.
   *
   * @var int
   */
  public static $openStackSecurityGroupRepeatCount = 2;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_REPEAT_COUNT.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesRepeatCount = 10;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_INBOUND.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesInbound = 0;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_OUTBOUND.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesOutbound = 1;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_MIX.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesMix = 2;

  /**
   * Create test data for cloud service provider (CloudConfig).
   *
   * @param int $max_count
   *   The max repeat count.
   * @param bool $ec2_api
   *   Whether EC2 API or not.
   *
   * @return array
   *   Test data.
   */
  protected function createOpenStackCloudConfigTestFormData($max_count = 1, $ec2_api = TRUE): array {
    $this->random = new Random();
    $cloud_context = $this->random->name(8, TRUE);

    // Input Fields.
    $data = [];
    for ($i = 0, $num = 1; $i < $max_count; $i++, $num++) {

      $data[$i] = [
        'name[0][value]'            => sprintf('config-entity-#%d-%s - %s', $num, $cloud_context, date('Y/m/d H:i:s')),
        'field_os_region[0][value]' => 'RegionOne',
      ];

      if ($ec2_api) {
        $data[$i]['field_use_openstack_ec2_api[value]'] = $ec2_api;
        $data[$i]['field_api_endpoint[0][value]'] = 'http://openstack.endpoint:8788';
        $data[$i]['field_access_key[0][value]'] = $this->random->name(40, TRUE);
        $data[$i]['field_secret_key[0][value]'] = $this->random->name(40, TRUE);
        $data[$i]['field_account_id[0][value]'] = '';
      }
      else {
        $data[$i]['field_api_endpoint[0][value]'] = 'http://openstack.endpoint';
        $data[$i]['field_username[0][value]'] = $this->random->name(8, TRUE);
        $data[$i]['field_password_wrapper'] = $this->random->name(16, TRUE);
        $data[$i]['field_domain_id[0][value]'] = $this->random->name(8, TRUE);
        $data[$i]['field_project_id[0][value]'] = $this->random->name(32, TRUE);
        $data[$i]['field_api_endpoint_port[0][value]'] = rand(1000, 9999);
        $data[$i]['field_compute_version[0][value]'] = $this->random->name(2, TRUE);
        $data[$i]['field_image_version[0][value]'] = $this->random->name(2, TRUE);
        $data[$i]['field_volume_version[0][value]'] = $this->random->name(2, TRUE);
        $data[$i]['field_network_version[0][value]'] = $this->random->name(2, TRUE);
      }
    }

    return $data;
  }

  /**
   * Create instance test data.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   test data array.
   */
  protected function createOpenStackInstanceTestFormData($repeat_count = 1): array {
    $data = [];

    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {

      $instance_name = sprintf('instance-entity #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(8, TRUE));

      // Input Fields.
      $data[$i] = [
        'name'                => $instance_name,
        'image_id'            => "ami-{$this->getRandomId()}",
        'min_count'           => $num,
        'max_count'           => $num * 2,
        'key_pair_name'       => "key_pair-${num}-{$this->random->name(8, TRUE)}",
        'is_monitoring'       => 0,
        'availability_zone'   => 'RegionOne',
        'security_groups[]'   => "security_group-$num-{$this->random->name(8, TRUE)}",
        'instance_type'       => 'm1.nano',
        'kernel_id'           => "aki-{$this->getRandomId()}",
        'ramdisk_id'          => "ari-{$this->getRandomId()}",
        'user_data'           => "User Data #${num}: {$this->random->name(64, TRUE)}",
        'tags[0][item_key]'   => 'Name',
        'tags[0][item_value]' => $instance_name,
      ];
    }

    return $data;
  }

  /**
   * Create cloud server template test data.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createOpenStackServerTemplateTestFormData($repeat_count = 1): array {
    $data = [];
    $random = $this->random;

    for ($i = 0, $num = 1, $instance_family = 3; $i < $repeat_count; $i++, $num++, $instance_family++) {

      // Input Fields.
      $data[] = [
        'cloud_context[0][value]' => $this->cloudContext,
        'name[0][value]' => "LaunchTemplate-{$random->name(16, TRUE)}",
        'field_description[0][value]' => "#$num: " . date('Y/m/d H:i:s - D M j G:i:s T Y')
        . ' - SimpleTest Launch Template Description - '
        . $random->name(32, TRUE),
        'field_test_only[value]' => '1',
        'field_os_availability_zone' => 'RegionOne',
        'field_monitoring[value]' => '1',
        'field_openstack_image_id' => "ami-{$this->getRandomId()}",
        'field_min_count[0][value]' => 1,
        'field_max_count[0][value]' => 1,
        'field_kernel_id[0][value]' => "aki-{$this->getRandomId()}",
        'field_ram[0][value]' => "ari-{$this->getRandomId()}",
        'field_openstack_security_group' => 1,
        'field_openstack_ssh_key' => 1,
        'field_openstack_network' => 1,
        'field_tags[0][item_key]' => "key_{$random->name(8, TRUE)}",
        'field_tags[0][item_value]' => "value_{$random->name(8, TRUE)}",
      ];
    }
    return $data;
  }

  /**
   * Create Floating IP test data.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param array $network_interfaces
   *   Network interfaces array.
   *
   * @return string[][]
   *   Floating IP array.
   */
  protected function createElasticIpTestFormData($repeat_count, array $network_interfaces = []): array {
    $data = [];
    // 3 times.
    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {
      // Input Fields.
      $data[$i] = [
        'name'   => sprintf('eip-random-data #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'domain' => 'standard',
        'network_border_group' => 'us-west-2',
        'floating_network_id' => !empty($network_interfaces[$i]) ? $network_interfaces[$i] : 'eni-' . $this->random->name(32, TRUE),
      ];
    }
    return $data;
  }

  /**
   * Create rules.
   *
   * @param int $rules_type
   *   The type of rules. Inbound | Outbound | Mixed.
   * @param string $edit_url
   *   The URL of security group edit form.
   * @param string $self_group_id
   *   The security group ID for rules.
   * @param int $security_group_rules_repeat_count
   *   The security group rules repeat count.
   * @param bool $openstack_ec2_api
   *   Whether EC2 service or REST service.
   *
   * @return array
   *   The rules created.
   *
   * @throws \Exception
   */
  protected function createRulesTestFormData($rules_type, $edit_url, $self_group_id = NULL, $security_group_rules_repeat_count = 1, $openstack_ec2_api = TRUE): array {
    $rules = [];
    $count = random_int(1, $security_group_rules_repeat_count);
    for ($i = 0; $i < $count; $i++) {
      $permissions = [
        [
          'source' => 'ip4',
          'cidr_ip' => Utils::getRandomCidr(),
          'from_port' => Utils::getRandomFromPort(),
          'to_port' => Utils::getRandomToPort(),
          'description' => $this->getRandomSecurityGroupDescription(),
        ],
        [
          'source' => 'ip4',
          'cidr_ip' => Utils::getRandomCidr(),
          'from_port' => Utils::getRandomFromPort(0, 0),
          'to_port' => Utils::getRandomToPort(),
          'description' => $this->getRandomSecurityGroupDescription(),
        ],
        [
          'source' => 'ip6',
          'cidr_ip_v6' => Utils::getRandomCidrV6(),
          'from_port' => Utils::getRandomFromPort(),
          'to_port' => Utils::getRandomToPort(),
          'description' => $this->getRandomSecurityGroupDescription(),
        ],
      ];

      if (!($openstack_ec2_api)) {
        foreach ($permissions as &$permission) {
          $permission['rule_id'] = $this->getRandomId();
        }
      }

      if ($rules_type === self::$openStackSecurityGroupRulesInbound
        || $rules_type === self::$openStackSecurityGroupRulesOutbound) {
        $type = $rules_type;
      }
      else {
        $types = [
          self::$openStackSecurityGroupRulesInbound,
          self::$openStackSecurityGroupRulesOutbound,
        ];
        $type = $types[array_rand($types)];
      }
      $rules[] = $permissions[array_rand($permissions)] + ['type' => $type];
    }

    // Post to form.
    $params = [];
    $inbound_index = 0;
    $outbound_index = 0;
    $rules_added = [];
    foreach ($rules ?: [] as $rule) {
      if ($rule['type'] === self::$openStackSecurityGroupRulesInbound) {
        $index = $inbound_index++;
        $prefix = 'ip_permission';
      }
      else {
        $index = $outbound_index++;
        $prefix = 'outbound_permission';
      }

      foreach ($rule ?: [] as $key => $value) {
        if ($key === 'type') {
          continue;
        }
        $params["${prefix}[${index}][${key}]"] = $value;
      }

      $rules_added[] = $rule;
      $this->updateRulesMockData($rules_added, self::$openStackSecurityGroupRulesOutbound);

      $this->drupalGet($edit_url);
      $this->submitForm($params, $this->t('Save'));
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($rule['from_port']);
    }

    // Confirm the values of edit form.
    $this->confirmRulesFormData($rules, $edit_url);

    return $rules;
  }

  /**
   * Create random instances data.
   *
   * @return array
   *   Random instances data.
   *
   * @throws \Exception
   */
  protected function createInstancesRandomTestFormData(): array {
    $instances = [];
    $count = random_int(1, 10);
    for ($i = 0, $num = 1; $i < $count; $i++, $num++) {
      $instances[] = [
        'InstanceId' => "i-{$this->getRandomId()}",
        'Name' => sprintf('instance-random-data #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $instances;
  }

  /**
   * Create random flavors.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @throws \Exception
   */
  protected function createFlavorsRandomTestFormData($repeat_count = 1): array {
    $flavors = [];
    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {
      $flavors[] = [
        'id' => $num,
        'name' => $this->random->name(8, TRUE),
      ];
    }

    return $flavors;
  }

}

==> cloud-3.2.0.tar.gz: cloud/modules/cloud_service_providers/openstack/tests/src/Traits/OpenStackTestFormDataTrait.php <==
<?php

namespace Drupal\Tests\openstack\Traits;

use Drupal\Component\Utility\Random;
use Drupal\Tests\aws_cloud\Traits\AwsCloudTestFormDataTrait;
use Drupal\Tests\cloud\Functional\Utils;

/**
 * The trait creating form data for openstack testing.
 */
trait OpenStackTestFormDataTrait {

  use AwsCloudTestFormDataTrait;

  /**
   * OPENSTACK_SECURITY_GROUP_REPEAT_COUNT.
   *
   * @var int
   */
  public static $openStackSecurityGroupRepeatCount = 2;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_REPEAT_COUNT.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesRepeatCount = 10;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_INBOUND.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesInbound = 0;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_OUTBOUND.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesOutbound = 1;

  /**
   * OPENSTACK_SECURITY_GROUP_RULES_MIX.
   *
   * @var int
   */
  public static $openStackSecurityGroupRulesMix = 2;

  /**
   * Create test data for cloud service provider (CloudConfig).
   *
   * @param int $max_count
   *   The max repeat count.
   * @param bool $ec2_api
   *   Whether EC2 API or not.
   *
   * @return array
   *   Test data.
   */
  protected function createOpenStackCloudConfigTestFormData($max_count = 1, $ec2_api = TRUE): array {
    $this->random = new Random();
    $cloud_context = $this->random->name(8, TRUE);

    // Input Fields.
    $data = [];
    for ($i = 0, $num = 1; $i < $max_count; $i++, $num++) {

      $data[$i] = [
        'name[0][value]'            => sprintf('config-entity-#%d-%s - %s', $num, $cloud_context, date('Y/m/d H:i:s')),
        'field_os_region[0][value]' => 'RegionOne',
      ];

      if ($ec2_api) {
        $data[$i]['field_use_openstack_ec2_api[value]'] = $ec2_api;
        $data[$i]['field_api_endpoint[0][value]'] = 'http://openstack.endpoint:8788';
        $data[$i]['field_access_key[0][value]'] = $this->random->name(40, TRUE);
        $data[$i]['field_secret_key[0][value]'] = $this->random->name(40, TRUE);
        $data[$i]['field_account_id[0][value]'] = '';
      }
      else {
        $data[$i]['field_api_endpoint[0][value]'] = 'http://openstack.endpoint';
        $data[$i]['field_username[0][value]'] = $this->random->name(8, TRUE);
        $data[$i]['field_password_wrapper'] = $this->random->name(16, TRUE);
        $data[$i]['field_domain_id[0][value]'] = $this->random->name(8, TRUE);
        $data[$i]['field_project_id[0][value]'] = $this->random->name(32, TRUE);
        $data[$i]['field_api_endpoint_port[0][value]'] = rand(1000, 9999);
        $data[$i]['field_compute_version[0][value]'] = $this->random->name(2, TRUE);
        $data[$i]['field_image_version[0][value]'] = $this->random->name(2, TRUE);
        $data[$i]['field_volume_version[0][value]'] = $this->random->name(2, TRUE);
        $data[$i]['field_network_version[0][value]'] = $this->random->name(2, TRUE);
      }
    }

    return $data;
  }

  /**
   * Create instance test data.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   test data array.
   */
  protected function createOpenStackInstanceTestFormData($repeat_count = 1): array {
    $data = [];

    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {

      $instance_name = sprintf('instance-entity #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(8, TRUE));

      // Input Fields.
      $data[$i] = [
        'name'                => $instance_name,
        'image_id'            => "ami-{$this->getRandomId()}",
        'min_count'           => $num,
        'max_count'           => $num * 2,
        'key_pair_name'       => "key_pair-${num}-{$this->random->name(8, TRUE)}",
        'is_monitoring'       => 0,
        'availability_zone'   => 'RegionOne',
        'security_groups[]'   => "security_group-$num-{$this->random->name(8, TRUE)}",
        'instance_type'       => 'm1.nano',
        'kernel_id'           => "aki-{$this->getRandomId()}",
        'ramdisk_id'          => "ari-{$this->getRandomId()}",
        'user_data'           => "User Data #${num}: {$this->random->name(64, TRUE)}",
        'tags[0][item_key]'   => 'Name',
        'tags[0][item_value]' => $instance_name,
      ];
    }

    return $data;
  }

  /**
   * Create Cloud launch template test data.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createOpenStackServerTemplateTestFormData($repeat_count = 1): array {
    $data = [];
    $random = $this->random;

    for ($i = 0, $num = 1, $instance_family = 3; $i < $repeat_count; $i++, $num++, $instance_family++) {

      // Input Fields.
      $data[] = [
        'cloud_context[0][value]' => $this->cloudContext,
        'name[0][value]' => "LaunchTemplate-{$random->name(16, TRUE)}",
        'field_description[0][value]' => "#$num: " . date('Y/m/d H:i:s - D M j G:i:s T Y')
        . ' - SimpleTest Launch Template Description - '
        . $random->name(32, TRUE),
        'field_test_only[value]' => '1',
        'field_os_availability_zone' => 'RegionOne',
        'field_monitoring[value]' => '1',
        'field_openstack_image_id' => "ami-{$this->getRandomId()}",
        'field_min_count[0][value]' => 1,
        'field_max_count[0][value]' => 1,
        'field_kernel_id[0][value]' => "aki-{$this->getRandomId()}",
        'field_ram[0][value]' => "ari-{$this->getRandomId()}",
        'field_openstack_security_group' => 1,
        'field_openstack_ssh_key' => 1,
        'field_openstack_network' => 1,
        'field_tags[0][item_key]' => "key_{$random->name(8, TRUE)}",
        'field_tags[0][item_value]' => "value_{$random->name(8, TRUE)}",
      ];
    }
    return $data;
  }

  /**
   * Create Floating IP test data.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param array $network_interfaces
   *   Network interfaces array.
   *
   * @return string[][]
   *   Floating IP array.
   */
  protected function createElasticIpTestFormData($repeat_count, array $network_interfaces = []): array {
    $data = [];
    // 3 times.
    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {
      // Input Fields.
      $data[$i] = [
        'name'   => sprintf('eip-random-data #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'domain' => 'standard',
        'network_border_group' => 'us-west-2',
        'floating_network_id' => !empty($network_interfaces[$i]) ? $network_interfaces[$i] : 'eni-' . $this->random->name(32, TRUE),
      ];
    }
    return $data;
  }

  /**
   * Create rules.
   *
   * @param int $rules_type
   *   The type of rules. Inbound | Outbound | Mixed.
   * @param string $edit_url
   *   The URL of security group edit form.
   * @param string $self_group_id
   *   The security group ID for rules.
   * @param int $security_group_rules_repeat_count
   *   The security group rules repeat count.
   * @param bool $openstack_ec2_api
   *   Whether EC2 service or REST service.
   *
   * @return array
   *   The rules created.
   *
   * @throws \Exception
   */
  protected function createRulesTestFormData($rules_type, $edit_url, $self_group_id = NULL, $security_group_rules_repeat_count = 1, $openstack_ec2_api = TRUE): array {
    $rules = [];
    $count = random_int(1, $security_group_rules_repeat_count);
    for ($i = 0; $i < $count; $i++) {
      $permissions = [
        [
          'source' => 'ip4',
          'cidr_ip' => Utils::getRandomCidr(),
          'from_port' => Utils::getRandomFromPort(),
          'to_port' => Utils::getRandomToPort(),
          'description' => $this->getRandomSecurityGroupDescription(),
        ],
        [
          'source' => 'ip4',
          'cidr_ip' => Utils::getRandomCidr(),
          'from_port' => Utils::getRandomFromPort(0, 0),
          'to_port' => Utils::getRandomToPort(),
          'description' => $this->getRandomSecurityGroupDescription(),
        ],
        [
          'source' => 'ip6',
          'cidr_ip_v6' => Utils::getRandomCidrV6(),
          'from_port' => Utils::getRandomFromPort(),
          'to_port' => Utils::getRandomToPort(),
          'description' => $this->getRandomSecurityGroupDescription(),
        ],
      ];

      if (!($openstack_ec2_api)) {
        foreach ($permissions as &$permission) {
          $permission['rule_id'] = $this->getRandomId();
        }
      }

      if ($rules_type === self::$openStackSecurityGroupRulesInbound
        || $rules_type === self::$openStackSecurityGroupRulesOutbound) {
        $type = $rules_type;
      }
      else {
        $types = [
          self::$openStackSecurityGroupRulesInbound,
          self::$openStackSecurityGroupRulesOutbound,
        ];
        $type = $types[array_rand($types)];
      }
      $rules[] = $permissions[array_rand($permissions)] + ['type' => $type];
    }

    // Post to form.
    $params = [];
    $inbound_index = 0;
    $outbound_index = 0;
    $rules_added = [];
    foreach ($rules ?: [] as $rule) {
      if ($rule['type'] === self::$openStackSecurityGroupRulesInbound) {
        $index = $inbound_index++;
        $prefix = 'ip_permission';
      }
      else {
        $index = $outbound_index++;
        $prefix = 'outbound_permission';
      }

      foreach ($rule ?: [] as $key => $value) {
        if ($key === 'type') {
          continue;
        }
        $params["${prefix}[${index}][${key}]"] = $value;
      }

      $rules_added[] = $rule;
      $this->updateRulesMockData($rules_added, self::$openStackSecurityGroupRulesOutbound);

      $this->drupalGet($edit_url);
      $this->submitForm($params, $this->t('Save'));
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($rule['from_port']);
    }

    // Confirm the values of edit form.
    $this->confirmRulesFormData($rules, $edit_url);

    return $rules;
  }

  /**
   * Create random instances data.
   *
   * @return array
   *   Random instances data.
   *
   * @throws \Exception
   */
  protected function createInstancesRandomTestFormData(): array {
    $instances = [];
    $count = random_int(1, 10);
    for ($i = 0, $num = 1; $i < $count; $i++, $num++) {
      $instances[] = [
        'InstanceId' => "i-{$this->getRandomId()}",
        'Name' => sprintf('instance-random-data #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $instances;
  }

  /**
   * Create random flavors.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @throws \Exception
   */
  protected function createFlavorsRandomTestFormData($repeat_count = 1): array {
    $flavors = [];
    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {
      $flavors[] = [
        'id' => $num,
        'name' => $this->random->name(8, TRUE),
      ];
    }

    return $flavors;
  }

  /**
   * Get the unused volume criteria options on OpenStack Settings.
   *
   * @return array
   *   Array of Unused volume criteria.
   */
  protected function getUnusedVolumeCriteriaOptions(): array {
    return [30, 60, 90, 180, 365];
  }

  /**
   * Create an array of random input data for Settings.
   *
   * @return array
   *   The array including random input data.
   */
  public function createOpenStackSettingsFormData($repeat_count = 1): array {
    $config_factory = \Drupal::configFactory();
    $config = $config_factory->getEditable('openstack.settings');

    $unused_volume_criteria = $this->getUnusedVolumeCriteriaOptions();

    $edit = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $edit[] = [
        'openstack_test_mode' => !$config->get('openstack_test_mode'),
        'openstack_unused_volume_criteria' => $unused_volume_criteria[array_rand($unused_volume_criteria)],
        'openstack_instance_terminate' => !$config->get('openstack_instance_terminate'),
        'openstack_update_resources_queue_cron_time' => random_int(1, 9999),
        'openstack_queue_limit' => random_int(1, 50),
      ];
    }

    return $edit;
  }

}
